;;
;; cdb.scm  -- cdb interface
;;
;; $Id: cdb.scm,v 1.2 2003/11/21 22:42:43 hos Exp $
;;

(define-module dbm.cdb
  (extend dbm)
  (export <cdb>
	  cdb-init
	  cdb-free cdb-freed?
	  cdb-read cdb-exists?
	  cdb-make-start
	  cdb-make-finish cdb-make-finished?
	  cdb-make-add cdb-make-exists?)
  )
(select-module dbm.cdb)
(dynamic-load "cdb")

;;
;; Initialize
;;

(define-class <cdb-meta> (<dbm-meta>)
  ())

(define-class <cdb> (<dbm>)
  ((cdb-file :accessor cdb-file-of :initform #f)
   (cdb-make-file :accessor cdb-make-file-of :initform #f)
   )
  :metaclass <cdb-meta>)

(define-method dbm-open ((self <cdb>))
  (next-method)
  (unless (slot-bound? self 'path)
	  (errorf "path must be set to open ndbm database"))
  (when (cdb-file-of self)
	(errorf "cdb ~S already opened" self))
  (case (slot-ref self 'rw-mode)
    ((:read)
     (let* ((path (slot-ref self 'path))
	    (fp (cdb-init path)))
       (slot-set! self 'cdb-file fp)
       self))
    ((:create)
     (let* ((path (slot-ref self 'path))
	    (fp (cdb-make-start path)))
       (slot-set! self 'cdb-make-file fp)
       self))
    (else
     (error "cdb is only be able to open :read or :create"))))

;;
;; close operation
;;

(define-method dbm-close ((self <cdb>))
  (let ((cdb (cdb-file-of self))
	(cdb-make (cdb-make-file-of self)))
    (cond (cdb
	   (cdb-free cdb))
	  (cdb-make
	   (cdb-make-finish cdb-make)))))

(define-method dbm-closed? ((self <cdb>))
  (let ((cdb (cdb-file-of self))
	(cdb-make (cdb-make-file-of self)))
    (cond (cdb
	   (cdb-freed? cdb))
	  (cdb-make
	   (cdb-make-finished? cdb-make)))))

;;
;; common operations
;;

(define-method dbm-get ((self <cdb>) key . args)
  (next-method)
  (let ((cdb (cdb-file-of self)))
    (if cdb
	(cond ((cdb-read (cdb-file-of self) (%dbm-k2s self key))
	       => (lambda (v) (%dbm-s2v self v)))
	      ((pair? args) (car args))     ;fall-back value
	      (else  (errorf "cdb: no data for key ~s in database ~s"
			     key (cdb-file-of self))))
	(error "cdb: open for write only" self))))

(define-method dbm-put! ((self <cdb>) key val)
  (next-method)
  (let ((cdb-make (cdb-make-file-of self)))
    (if cdb-make
	(cdb-make-add cdb-make key val)
	(error "cdb: open for read only" self))))

(define-method dbm-exists? ((self <cdb>) key)
  (next-method)
  (let ((cdb (cdb-file-of self)))
    (if cdb
	(cdb-exists? cdb (%dbm-k2s self key))
	(error "cdb: open for write only" self))))

(define-method dbm-delete! ((self <cdb>) key)
  (next-method)
  (error "cdb: delete is not supported" self))

;;
;; Metaoperations
;;

(define-method dbm-db-exists? ((class <cdb-meta>) name)
  (file-exists? name))

(define-method dbm-db-remove ((class <cdb-meta>) name)
  (sys-unlink name))

(define-method dbm-db-copy ((class <cdb-meta>) from to . keys)
  (apply copy-file from to :safe #t keys))

(define-method dbm-db-rename ((class <cdb-meta>) from to . keys)
  (apply move-file from to :safe #t keys))

(provide "dbm/cdb")
